------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                               J X _ C H 8                                --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 1998-2013, AdaCore                     --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT; see file COPYING3.  If not, go to --
-- http://www.gnu.org/licenses for a complete copy of the license.          --
--                                                                          --
-- The GNAT Ada tool chain for the JVM and .NET platforms is  maintained by --
-- AdaCore - http://www.adacore.com                                         --
--                                                                          --
------------------------------------------------------------------------------

--  This package implements translation of renaming declarations

with J_Stack;
with Types;   use Types;

package Jx_Ch8 is

   package JVM_Scope_Stack is new J_Stack (Entity_Id, 50);
   --  Used by Jx_Drive.Translate to store entities of packages and subprograms
   --  processed as part of traversing the tree generated by the frontend. That
   --  is, entities associated with type declarations are not stored. Its main
   --  use is to provide complementary information to the backend about the
   --  context of the emitted code. For example, it permits the backend to
   --  diferentiate an implicit method declaration associated with a call
   --  from a method declaration found in the sources.

   function JVM_Current_Scope return Entity_Id;
   --  Get entity representing current scope

   procedure Translate_Object_Renaming (Obj_Renaming : Node_Id);
   --  Translates an object renaming declaration into a JVM object reference,
   --  if needed, and evaluates the name given in the renaming declaration. In
   --  some cases of scalar object renamings, the evaluation of the prefix of a
   --  compound name will be saved, and in the case of simple renamings of
   --  scalar objects whose name is not vulnerable to side effects nothing need
   --  be saved (and a later reference can simply reevaluate the name).

end Jx_Ch8;
